from charmhelpers.core import hookenv
from charms.reactive import when, when_all, when_not, set_flag
import os
import shutil
from subprocess import call, check_call, CalledProcessError


@when_not('fuse-nfs.installed')
@when('apt.installed.libfuse-dev')
def install_fuse_nfs_client():
    # Do your setup here.
    #
    # If your charm has other dependencies before it can install,
    # add those as @when() clauses above., or as additional @when()
    # decorated handlers below
    #
    # See the following for information about reactive charms:
    #
    #  * https://jujucharms.com/docs/devel/developer-getting-started
    #  * https://github.com/juju-solutions/layer-basic#overview
    #
    try:
        check_call(['which', 'fuse-nfs'])
        set_flag('fuse-nfs.installed')
    except CalledProcessError:
        try:
            hookenv.status_set('maintenance', 'Cloning fuse-nfs')
            check_call(["git",
                        "clone",
                        hookenv.config().get("source"),
                        "/tmp/fuse-nfs"])
            os.chdir("/tmp/fuse-nfs")
            hookenv.status_set('maintenance', 'Building fuse-nfs')
            check_call(["./setup.sh"])
            check_call(["./configure"])
            check_call(["make"])
            hookenv.status_set('maintenance', 'Installing fuse-nfs')
            check_call(["make", "install"])
            set_flag('fuse-nfs.installed')
        except CalledProcessError as e:
            hookenv.log("Error setting up fuse-nfs: {}".format(e))
            raise
        finally:
            shutil.rmtree('/tmp/fuse-nfs')
    configure_mountpoint()


@when_all('config.set.nfs-host',
          'config.set.nfs-share',
          'config.set.mountpoint',
          'fuse-nfs.installed')
def configure_mountpoint():
    config = hookenv.config()
    old_mountpoint = config.previous('mountpoint')
    if old_mountpoint:
        remove_mount(old_mountpoint)
    host = config.get('nfs-host')
    share = config.get('nfs-share')
    mountpoint = config.get('mountpoint')
    if host and share and mountpoint:
        add_mount(
            host,
            share,
            mountpoint
        )
        hookenv.status_set('active', '{}:{} is mounted at {}'.format(
            host,
            share,
            mountpoint
        ))
    else:
        hookenv.status_set(
            'blocked',
            'need configuration: {{ host: {}, share: {}, mountpoint: {}}}'
            .format(host, share, mountpoint))


def remove_mount(mountpoint):
    """Unmount the specified fuse mount"""
    call(["fusermount", "-u", mountpoint])


def add_mount(host, remote_path, mountpoint):
    """Mount the specified NFS path"""
    try:
        os.mkdir(mountpoint)
    except OSError:  # mkdir throws OSError when directory exists
        pass
    call(["fuse-nfs",
          "-n", "nfs://{}{}".format(host, remote_path),
          "-m", mountpoint, "--allow_other"])
